package ru.dbotthepony.kstarbound.client

import ru.dbotthepony.kstarbound.client.gl.GLStateTracker
import ru.dbotthepony.kstarbound.client.render.entity.EntityRenderer
import ru.dbotthepony.kstarbound.world.Chunk
import ru.dbotthepony.kstarbound.world.ChunkPos
import ru.dbotthepony.kstarbound.world.entities.Entity

/**
 * Псевдо zPos у фоновых тайлов
 *
 * Добавление этого числа к zPos гарантирует, что фоновые тайлы будут отрисованы
 * первыми (на самом дальнем плане)
 */
const val Z_LEVEL_BACKGROUND = 60000
const val Z_LEVEL_LIQUID = 10000

class ClientChunk(world: ClientWorld, pos: ChunkPos) : Chunk<ClientWorld, ClientChunk>(world, pos){
	val state: GLStateTracker get() = world.client.gl

	override fun foregroundChanges(cell: Cell) {
		super.foregroundChanges(cell)

		world.forEachRenderRegion(cell) {
			it.foreground.isDirty = true
		}
	}

	override fun backgroundChanges(cell: Cell) {
		super.backgroundChanges(cell)

		world.forEachRenderRegion(cell) {
			it.background.isDirty = true
		}
	}

	override fun liquidChanges(cell: Cell) {
		super.liquidChanges(cell)

		world.forEachRenderRegion(cell) {
			it.liquidIsDirty = true
		}
	}

	private val entityRenderers = HashMap<Entity, EntityRenderer>()

	override fun onEntityAdded(entity: Entity) {
		entityRenderers[entity] = EntityRenderer.getRender(world.client, entity, this)
	}

	override fun onEntityTransferedToThis(entity: Entity, otherChunk: ClientChunk) {
		val renderer = otherChunk.entityRenderers[entity] ?: throw IllegalStateException("$otherChunk has no renderer for $entity!")
		entityRenderers[entity] = renderer
		renderer.chunk = this
	}

	override fun onEntityTransferedFromThis(entity: Entity, otherChunk: ClientChunk) {
		entityRenderers.remove(entity)
	}

	override fun onEntityRemoved(entity: Entity) {
		entityRenderers.remove(entity)
	}
}
